/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "cmd.h"
#include "pmhash.h"
#include "token.h"
#include <errno.h>
#include <stdlib.h>

struct pmhash *cmds;

int cmd_init(void)
{
	cmds = pmhash_new();
	if (!cmds)
		return -errno;
	return 0;
}

int cmd_add(struct cmd *cmd)
{
	return pmhash_add(&cmds, cmd->name, cmd);
}

int cmd_run(struct declaracao *dec, char *line)
{
	char **args = tokens_new(line);
	struct cmd *cmd;
	int r;
	int argc;
	if (!args || !args[0])
		return -errno;
	for (argc = 0; args[argc]; argc++);
	cmd = pmhash_get(cmds, args[0]);
	if (!cmd || !cmd->run) {
		tokens_free(args);
		return -EINVAL;
	}
	r = cmd->run(dec, args, argc);
	tokens_free(args);
	return r;
}

struct cmd * cmd_next(void **iter)
{
	return pmhash_next(cmds, iter);
}
